@@ -10,7 +10,7 @@ open Ctypes
 type window = unit ptr
 let window : window typ = ptr void
 
-module Bindings (F : Cstubs.FOREIGN) =
+module Bindings (F : Ctypes.FOREIGN) =
 struct
   open F
 
@@ -47,20 +47,3 @@ struct
   let cbreak =
     foreign "cbreak" (void @-> (returning void))
 end
-
-let c_headers = "#include <ncurses.h>"
-
-let main () =
-  let ml_out = open_out "examples/ncurses/stub-generation/ncurses_generated.ml" in
-  let c_out = open_out "examples/ncurses/stub-generation/ncurses_stubs.c" in
-  let c_fmt = Format.formatter_of_out_channel c_out in
-  let ml_fmt = Format.formatter_of_out_channel ml_out in
-  Format.fprintf c_fmt "%s@\n" c_headers;
-  Cstubs.write_c c_fmt ~prefix:"ncurses_stub_" (module Bindings);
-  Cstubs.write_ml ml_fmt ~prefix:"ncurses_stub_" (module Bindings);
-  Format.pp_print_flush ml_fmt ();
-  Format.pp_print_flush c_fmt ();
-  close_out ml_out;
-  close_out c_out
-
-let () = main ()
